@@ -73,12 +73,14 @@
      VAR_REQUEST_XFR = 289,
      VAR_NOTIFY = 290,
      VAR_PROVIDE_XFR = 291,
-     VAR_KEY = 292,
-     VAR_ALGORITHM = 293,
-     VAR_SECRET = 294,
-     VAR_AXFR = 295,
-     VAR_VERBOSITY = 296,
-     VAR_HIDE_VERSION = 297
+     VAR_OUTGOING_INTERFACE = 292,
+     VAR_KEY = 293,
+     VAR_ALGORITHM = 294,
+     VAR_SECRET = 295,
+     VAR_AXFR = 296,
+     VAR_UDP = 297,
+     VAR_VERBOSITY = 298,
+     VAR_HIDE_VERSION = 299
    };
 #endif
 /* Tokens.  */
@@ -116,12 +118,14 @@
 #define VAR_REQUEST_XFR 289
 #define VAR_NOTIFY 290
 #define VAR_PROVIDE_XFR 291
-#define VAR_KEY 292
-#define VAR_ALGORITHM 293
-#define VAR_SECRET 294
-#define VAR_AXFR 295
-#define VAR_VERBOSITY 296
-#define VAR_HIDE_VERSION 297
+#define VAR_OUTGOING_INTERFACE 292
+#define VAR_KEY 293
+#define VAR_ALGORITHM 294
+#define VAR_SECRET 295
+#define VAR_AXFR 296
+#define VAR_UDP 297
+#define VAR_VERBOSITY 298
+#define VAR_HIDE_VERSION 299
 
 
 
@@ -133,7 +137,7 @@ typedef union YYSTYPE
 	char*	str;
 }
 /* Line 1489 of yacc.c.  */
-#line 137 "configparser.h"
+#line 141 "configparser.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
