@@ -136,21 +136,21 @@ read_rrset(namedb_type *db,
 	uint16_t type;
 	uint16_t klass;
 	uint32_t soa_minimum;
-	
+
 	owner = read_domain(db, domain_count, domains);
 	if (!owner)
 		return NULL;
 
 	rrset = (rrset_type *) region_alloc(db->region, sizeof(rrset_type));
-			     
+
 	rrset->zone = read_zone(db, zone_count, zones);
 	if (!rrset->zone)
 		return NULL;
-	
+
 	if (fread(&type, sizeof(type), 1, db->fd) != 1)
 		return NULL;
 	type = ntohs(type);
-	
+
 	if (fread(&klass, sizeof(klass), 1, db->fd) != 1)
 		return NULL;
 	klass = ntohs(klass);
@@ -162,14 +162,14 @@ read_rrset(namedb_type *db,
 		db->region, rrset->rr_count * sizeof(rr_type));
 
 	assert(rrset->rr_count > 0);
-	
+
 	for (i = 0; i < rrset->rr_count; ++i) {
 		rr_type *rr = &rrset->rrs[i];
 
 		rr->owner = owner;
 		rr->type = type;
 		rr->klass = klass;
-		
+
 		if (fread(&rr->rdata_count, sizeof(rr->rdata_count), 1, db->fd) != 1)
 			return NULL;
 		rr->rdata_count = ntohs(rr->rdata_count);
@@ -191,10 +191,10 @@ read_rrset(namedb_type *db,
 	if (rrset_rrtype(rrset) == TYPE_SOA) {
 		assert(owner == rrset->zone->apex);
 		rrset->zone->soa_rrset = rrset;
-		
+
 		/* BUG #103 add another soa with a tweaked ttl */
 		rrset->zone->soa_nx_rrset = region_alloc(db->region, sizeof(rrset_type));
-		rrset->zone->soa_nx_rrset->rrs = 
+		rrset->zone->soa_nx_rrset->rrs =
 			region_alloc(db->region, rrset->rr_count * sizeof(rr_type));
 
 		memcpy(rrset->zone->soa_nx_rrset->rrs, rrset->rrs, sizeof(rr_type));
@@ -208,8 +208,8 @@ read_rrset(namedb_type *db,
 		memcpy(&soa_minimum, rdata_atom_data(rrset->rrs->rdatas[6]),
 				rdata_atom_size(rrset->rrs->rdatas[6]));
 		if (rrset->rrs->ttl > ntohl(soa_minimum)) {
-			rrset->zone->soa_nx_rrset->rrs[0].ttl = ntohl(soa_minimum); 
-		} 
+			rrset->zone->soa_nx_rrset->rrs[0].ttl = ntohl(soa_minimum);
+		}
 
 	} else if (owner == rrset->zone->apex
 		   && rrset_rrtype(rrset) == TYPE_NS)
@@ -227,7 +227,6 @@ read_rrset(namedb_type *db,
 		}
 	}
 #endif
-	
 	return rrset;
 }
 
@@ -255,19 +254,19 @@ namedb_open (const char *filename, nsd_options_t* opt, size_t num_children)
 	 * returning.
 	 */
 	region_type *temp_region;
-	
+
 	uint32_t dname_count;
 	domain_type **domains;	/* Indexed by domain number.  */
 
 	uint32_t zone_count;
 	zone_type **zones;	/* Indexed by zone number.  */
-	
+
 	uint32_t i;
 	uint32_t rrset_count = 0;
 	uint32_t rr_count = 0;
 
 	rrset_type *rrset;
-	
+
 	DEBUG(DEBUG_DBACCESS, 2,
 	      (LOG_INFO, "sizeof(namedb_type) = %lu\n", (unsigned long) sizeof(namedb_type)));
 	DEBUG(DEBUG_DBACCESS, 2,
@@ -283,7 +282,7 @@ namedb_open (const char *filename, nsd_options_t* opt, size_t num_children)
 	DEBUG(DEBUG_DBACCESS, 2,
 	      (LOG_INFO, "sizeof(rbnode_t) = %lu\n", (unsigned long) sizeof(rbnode_t)));
 
-	db_region = region_create_custom(xalloc, free, DEFAULT_CHUNK_SIZE, 
+	db_region = region_create_custom(xalloc, free, DEFAULT_CHUNK_SIZE,
 		DEFAULT_LARGE_OBJECT_SIZE, DEFAULT_INITIAL_CLEANUP_SIZE, 1);
 	db = (namedb_type *) region_alloc(db_region, sizeof(struct namedb));
 	db->region = db_region;
@@ -294,6 +293,13 @@ namedb_open (const char *filename, nsd_options_t* opt, size_t num_children)
 	db->crc = 0xffffffff;
 	db->diff_skip = 0;
 
+	if (gettimeofday(&(db->diff_timestamp), NULL) != 0) {
+		log_msg(LOG_ERR, "unable to load %s: cannot initialize"
+				 "timestamp", db->filename);
+		region_destroy(db_region);
+                return NULL;
+        }
+
 	/* Open it... */
 	db->fd = fopen(db->filename, "r");
 	if (db->fd == NULL) {
@@ -353,9 +359,12 @@ namedb_open (const char *filename, nsd_options_t* opt, size_t num_children)
 		zones[i]->dirty = region_alloc(db->region, sizeof(uint8_t)*num_children);
 		memset(zones[i]->dirty, 0, sizeof(uint8_t)*num_children);
 		if(!zones[i]->opts) {
-			log_msg(LOG_ERR, "cannot load database. Zone %s in db %s, but not \
-in config file (might happen if you edited the config file). Please \
-rebuild database and start again.", dname_to_string(dname, NULL), db->filename);
+			log_msg(LOG_ERR, "cannot load database. Zone %s in db "
+					 "%s, but not in config file (might "
+					 "happen if you edited the config "
+					 "file). Please rebuild database and "
+					 "start again.",
+				dname_to_string(dname, NULL), db->filename);
 			region_destroy(dname_region);
 			region_destroy(temp_region);
 			namedb_close(db);
@@ -364,7 +373,7 @@ rebuild database and start again.", dname_to_string(dname, NULL), db->filename);
 
 		region_free_all(dname_region);
 	}
-	
+
 	if (!read_size(db, &dname_count)) {
 		log_msg(LOG_ERR, "corrupted database: %s", db->filename);
 		region_destroy(dname_region);
@@ -375,7 +384,7 @@ rebuild database and start again.", dname_to_string(dname, NULL), db->filename);
 
 	DEBUG(DEBUG_DBACCESS, 1,
 	      (LOG_INFO, "Retrieving %lu domain names\n", (unsigned long) dname_count));
-	
+
 	domains = (domain_type **) region_alloc(
 		temp_region, dname_count * sizeof(domain_type *));
 	for (i = 0; i < dname_count; ++i) {
@@ -390,14 +399,14 @@ rebuild database and start again.", dname_to_string(dname, NULL), db->filename);
 		domains[i] = domain_table_insert(db->domains, dname);
 		region_free_all(dname_region);
 	}
-	
+
 	region_destroy(dname_region);
 
 #ifndef NDEBUG
 	fprintf(stderr, "database region after loading domain names: ");
 	region_dump_stats(db->region, stderr);
 	fprintf(stderr, "\n");
-#endif	
+#endif
 
 	while ((rrset = read_rrset(db, dname_count, domains, zone_count, zones))) {
 		++rrset_count;
@@ -407,11 +416,11 @@ rebuild database and start again.", dname_to_string(dname, NULL), db->filename);
 	DEBUG(DEBUG_DBACCESS, 1,
 	      (LOG_INFO, "Retrieved %lu RRs in %lu RRsets\n",
 	       (unsigned long) rr_count, (unsigned long) rrset_count));
-	
+
 	region_destroy(temp_region);
-	
+
 	if ((db->crc_pos = ftello(db->fd)) == -1) {
-		log_msg(LOG_ERR, "ftello %s failed: %s", 
+		log_msg(LOG_ERR, "ftello %s failed: %s",
 			db->filename, strerror(errno));
 		namedb_close(db);
 		return NULL;
@@ -434,7 +443,7 @@ rebuild database and start again.", dname_to_string(dname, NULL), db->filename);
 	fprintf(stderr, "database region after loading database: ");
 	region_dump_stats(db->region, stderr);
 	fprintf(stderr, "\n");
-#endif	
+#endif
 
 	return db;
 }
