@@ -12,6 +12,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <limits.h>
+#include "tsig.h"
 #include "options.h"
 #include "util.h"
 #include "dname.h"
@@ -90,7 +91,7 @@ usage(void)
 	fprintf(stderr, "usage: nsd-checkconf [-v|-h] [-o option] [-z zonename]\n");
 	fprintf(stderr, "                     [-s keyname] <configfilename>\n");
 	fprintf(stderr, "       Checks NSD configuration file for errors.\n");
-	fprintf(stderr, "       Version %s. Report bugs to <%s>.\n\n", 
+	fprintf(stderr, "       Version %s. Report bugs to <%s>.\n\n",
 		PACKAGE_VERSION, PACKAGE_BUGREPORT);
 	fprintf(stderr, "Use with a configfile as argument to check syntax.\n");
 	fprintf(stderr, "Use with -o, -z or -s options to query the configuration.\n\n");
@@ -102,12 +103,12 @@ usage(void)
 	exit(1);
 }
 
-static void 
+static void
 print_string_var(const char* varname, const char* value)
 {
 	if (!value) {
 		printf("\t#%s\n", varname);
-	} else { 
+	} else {
 		printf("\t%s \"%s\"\n", varname, value);
 	}
 }
@@ -121,7 +122,7 @@ quote(const char *v)
 		printf("%s\n", v);
 }
 
-static void 
+static void
 quote_acl(acl_options_t* acl)
 {
 	while(acl)
@@ -133,7 +134,7 @@ quote_acl(acl_options_t* acl)
 	}
 }
 
-static void 
+static void
 print_acl(const char* varname, acl_options_t* acl)
 {
 	while(acl)
@@ -141,6 +142,8 @@ print_acl(const char* varname, acl_options_t* acl)
 		printf("\t%s ", varname);
 		if(acl->use_axfr_only)
 			printf("AXFR ");
+		if(acl->allow_udp)
+			printf("UDP ");
 		printf("%s %s\n", acl->ip_address_spec,
 			acl->nokey?"NOKEY":(acl->blocked?"BLOCKED":
 			(acl->key_name?acl->key_name:"(null)")));
@@ -220,6 +223,7 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 				ZONE_GET_ACL(provide_xfr, o);
 				ZONE_GET_ACL(allow_notify, o);
 				ZONE_GET_ACL(notify, o);
+				ZONE_GET_ACL(outgoing_interface, o);
 				printf("Zone option not handled: %s %s\n", z, o);
 				exit(1);
 			}
@@ -262,7 +266,7 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 	}
 }
 
-void 
+void
 config_test_print_server(nsd_options_t* opt)
 {
 	ip_address_option_t* ip;
@@ -311,11 +315,12 @@ config_test_print_server(nsd_options_t* opt)
 		print_acl("request-xfr:", zone->request_xfr);
 		print_acl("notify:", zone->notify);
 		print_acl("provide-xfr:", zone->provide_xfr);
+		print_acl("outgoing-interface:", zone->outgoing_interface);
 	}
-	
+
 }
 
-static int 
+static int
 additional_checks(nsd_options_t* opt, const char* filename)
 {
 	ip_address_option_t* ip = opt->ip_addresses;
@@ -362,9 +367,10 @@ additional_checks(nsd_options_t* opt, const char* filename)
 			fprintf(stderr, "%s: cannot base64 decode tsig secret: for key %s.\n", filename, key->name);
 			errors ++;
 		}
-		if(strcmp(key->algorithm, "hmac-md5") != 0)
+		if(tsig_get_algorithm_by_name(key->algorithm) != NULL)
 		{
-			fprintf(stderr, "%s: bad tsig algorithm: for key %s.\n", filename, key->name);
+			fprintf(stderr, "%s: bad tsig algorithm %s: for key \
+%s.\n", filename, key->algorithm, key->name);
 			errors ++;
 		}
 	}
@@ -372,7 +378,7 @@ additional_checks(nsd_options_t* opt, const char* filename)
 #ifndef BIND8_STATS
 	if(opt->statistics > 0)
 	{
-		fprintf(stderr, "%s: 'statistics: %d' but BIND 8 statistics feature not enabled.\n", 
+		fprintf(stderr, "%s: 'statistics: %d' but BIND 8 statistics feature not enabled.\n",
 			filename, opt->statistics);
 		errors ++;
 	}
@@ -380,7 +386,7 @@ additional_checks(nsd_options_t* opt, const char* filename)
 #ifndef HAVE_CHROOT
 	if(opt->chroot != 0)
 	{
-		fprintf(stderr, "%s: chroot %s given. chroot not supported on this platform.\n", 
+		fprintf(stderr, "%s: chroot %s given. chroot not supported on this platform.\n",
 			filename, opt->chroot);
 		errors ++;
 	}
@@ -397,45 +403,45 @@ additional_checks(nsd_options_t* opt, const char* filename)
                 int l = strlen(opt->chroot);
 
                 if (strncmp(opt->chroot, opt->pidfile, l) != 0) {
-			fprintf(stderr, "%s: pidfile %s is not relative to chroot %s.\n", 
+			fprintf(stderr, "%s: pidfile %s is not relative to chroot %s.\n",
 				filename, opt->pidfile, opt->chroot);
 			errors ++;
-                } 
+                }
 		if (strncmp(opt->chroot, opt->database, l) != 0) {
-			fprintf(stderr, "%s: databasefile %s is not relative to chroot %s.\n", 
+			fprintf(stderr, "%s: databasefile %s is not relative to chroot %s.\n",
 				filename, opt->database, opt->chroot);
 			errors ++;
                 }
 		if (strncmp(opt->chroot, opt->difffile, l) != 0) {
-			fprintf(stderr, "%s: difffile %s is not relative to chroot %s.\n", 
+			fprintf(stderr, "%s: difffile %s is not relative to chroot %s.\n",
 				filename, opt->difffile, opt->chroot);
 			errors ++;
                 }
 		if (strncmp(opt->chroot, opt->xfrdfile, l) != 0) {
-			fprintf(stderr, "%s: xfrdfile %s is not relative to chroot %s.\n", 
+			fprintf(stderr, "%s: xfrdfile %s is not relative to chroot %s.\n",
 				filename, opt->xfrdfile, opt->chroot);
 			errors ++;
                 }
         }
 	if (atoi(opt->port) <= 0) {
-		fprintf(stderr, "%s: port number '%s' is not a positive number.\n", 
+		fprintf(stderr, "%s: port number '%s' is not a positive number.\n",
 			filename, opt->port);
 		errors ++;
 	}
 	if(errors != 0) {
 		fprintf(stderr, "%s: %d semantic errors in %d zones, %d keys.\n",
-			filename, errors, (int)nsd_options_num_zones(opt), 
+			filename, errors, (int)nsd_options_num_zones(opt),
 			(int)opt->numkeys);
 	}
-	
+
 	return (errors == 0);
 }
 
-int 
+int
 main(int argc, char* argv[])
 {
 	int c;
-	int verbose = 0; 
+	int verbose = 0;
 	const char * conf_opt = NULL; /* what option do you want? Can be NULL -> print all */
 	const char * conf_zone = NULL; /* what zone are we talking about */
 	const char * conf_key = NULL; /* what key is needed */
@@ -478,9 +484,9 @@ main(int argc, char* argv[])
 		config_print_zone(options, conf_key, underscore(conf_opt), conf_zone);
 	} else {
 		if (verbose) {
-			printf("# Read file %s: %d zones, %d keys.\n", 
-				configfile, 
-				(int)nsd_options_num_zones(options), 
+			printf("# Read file %s: %d zones, %d keys.\n",
+				configfile,
+				(int)nsd_options_num_zones(options),
 				(int)options->numkeys);
 			config_test_print_server(options);
 		}
