@@ -34,7 +34,7 @@ struct nsd_options {
 	/* list of keys defined */
 	key_options_t* keys;
 	size_t numkeys;
-	
+
 	/* list of ip adresses to bind to (or NULL for all) */
 	ip_address_option_t* ip_addresses;
 
@@ -57,8 +57,8 @@ struct nsd_options {
 	const char* difffile;
 	const char* xfrdfile;
 	int xfrd_reload_timeout;
-	
-	region_type* region; 
+
+	region_type* region;
 };
 
 struct ipaddress_option {
@@ -80,6 +80,7 @@ struct zone_options {
 	acl_options_t* request_xfr;
 	acl_options_t* notify;
 	acl_options_t* provide_xfr;
+	acl_options_t* outgoing_interface;
 };
 
 union acl_addr_storage {
@@ -99,6 +100,7 @@ struct acl_options {
 
 	/* options */
 	uint8_t use_axfr_only;
+	uint8_t allow_udp;
 
 	/* ip address range */
 	const char* ip_address_spec;
@@ -148,6 +150,7 @@ struct config_parser_state {
 	acl_options_t* current_request_xfr;
 	acl_options_t* current_notify;
 	acl_options_t* current_provide_xfr;
+	acl_options_t* current_outgoing_interface;
 };
 
 extern config_parser_state_t* cfg_parser;
@@ -170,10 +173,10 @@ key_options_t* key_options_find(nsd_options_t* opt, const char* name);
 /* tsig must be inited, adds all keys in options to tsig. */
 void key_options_tsig_add(nsd_options_t* opt);
 
-/* check acl list, acl number that matches if passed(0..), 
+/* check acl list, acl number that matches if passed(0..),
  * or failure (-1) if dropped */
 /* the reason why (the acl) is returned too (or NULL) */
-int acl_check_incoming(acl_options_t* acl, struct query* q, 
+int acl_check_incoming(acl_options_t* acl, struct query* q,
 	acl_options_t** reason);
 int acl_addr_matches(acl_options_t* acl, struct query* q);
 int acl_key_matches(acl_options_t* acl, struct query* q);
