@@ -99,7 +99,7 @@ struct tcp_handler_data
 	 */
 	size_t              tcp_accept_handler_count;
 	netio_handler_type *tcp_accept_handlers;
-	
+
 	/*
 	 * The query_state is used to remember if we are performing an
 	 * AXFR, if we're done processing, or if we should discard the
@@ -192,9 +192,11 @@ delete_child_pid(struct nsd *nsd, pid_t pid)
 		if (nsd->children[i].pid == pid) {
 			nsd->children[i].pid = 0;
 			if(!nsd->children[i].need_to_exit) {
-				if(nsd->children[i].child_fd > 0) close(nsd->children[i].child_fd);
+				if(nsd->children[i].child_fd > 0)
+					close(nsd->children[i].child_fd);
 				nsd->children[i].child_fd = -1;
-				if(nsd->children[i].handler) nsd->children[i].handler->fd = -1;
+				if(nsd->children[i].handler)
+					nsd->children[i].handler->fd = -1;
 			}
 			return i;
 		}
@@ -300,7 +302,7 @@ static void set_bind8_alarm(struct nsd* nsd)
 #endif
 
 static void
-cleanup_dname_compression_tables(void *ptr) 
+cleanup_dname_compression_tables(void *ptr)
 {
 	free(ptr);
 	compressed_dname_offsets = NULL;
@@ -315,7 +317,7 @@ initialize_dname_compression_tables(struct nsd *nsd)
 	if(compression_table_capacity < needed) {
 		compressed_dname_offsets = (uint16_t *) xalloc(
 			needed * sizeof(uint16_t));
-		region_add_cleanup(nsd->db->region, cleanup_dname_compression_tables, 
+		region_add_cleanup(nsd->db->region, cleanup_dname_compression_tables,
 			compressed_dname_offsets);
 		compression_table_capacity = needed;
 		compression_table_size=domain_table_count(nsd->db->domains)+1;
@@ -387,7 +389,7 @@ server_init(struct nsd *nsd)
 					strerror(errno));
 				return -1;
 			}
-# endif
+#endif
 		}
 #endif
 		/* set it nonblocking */
@@ -399,7 +401,7 @@ server_init(struct nsd *nsd)
 
 		/* Bind it... */
 		if (bind(nsd->udp[i].s, (struct sockaddr *) nsd->udp[i].addr->ai_addr, nsd->udp[i].addr->ai_addrlen) != 0) {
-			log_msg(LOG_ERR, "can't bind the socket: %s", strerror(errno));
+			log_msg(LOG_ERR, "can't bind udp socket: %s", strerror(errno));
 			return -1;
 		}
 	}
@@ -447,7 +449,7 @@ server_init(struct nsd *nsd)
 
 		/* Bind it... */
 		if (bind(nsd->tcp[i].s, (struct sockaddr *) nsd->tcp[i].addr->ai_addr, nsd->tcp[i].addr->ai_addrlen) != 0) {
-			log_msg(LOG_ERR, "can't bind the socket: %s", strerror(errno));
+			log_msg(LOG_ERR, "can't bind tcp socket: %s", strerror(errno));
 			return -1;
 		}
 
@@ -475,7 +477,7 @@ server_init(struct nsd *nsd)
 			log_msg(LOG_ERR, "unable to chroot: %s", strerror(errno));
 			return -1;
 		}
-		DEBUG(DEBUG_IPC,1, (LOG_INFO, "changed root directory to %s", 
+		DEBUG(DEBUG_IPC,1, (LOG_INFO, "changed root directory to %s",
 			nsd->chrootdir));
 
 	}
@@ -495,21 +497,25 @@ server_init(struct nsd *nsd)
 
 	/* Drop the permissions */
 	if (setgid(nsd->gid) != 0 || setuid(nsd->uid) !=0) {
-		log_msg(LOG_ERR, "unable to drop user privileges: %s", strerror(errno));
+		log_msg(LOG_ERR, "unable to drop user privileges: %s",
+			strerror(errno));
 		unlink(nsd->pidfile);
 		return -1;
 	}
 
 	/* Open the database... */
 	if ((nsd->db = namedb_open(nsd->dbfile, nsd->options, nsd->child_count)) == NULL) {
+		log_msg(LOG_ERR, "unable to open the database %s: %s",
+			nsd->dbfile, strerror(errno));
 		unlink(nsd->pidfile);
 		return -1;
 	}
 
-
+	/* Read diff file */
 	if(!diff_read_file(nsd->db, nsd->options, NULL, nsd->child_count))
 	{
-		log_msg(LOG_ERR, "The diff file contains errors. Will continue without it");
+		log_msg(LOG_ERR, "The diff file contains errors. Will continue "
+						 "without it");
 	}
 #ifdef NSEC3
 	prehash(nsd->db, 0);
@@ -579,7 +585,7 @@ server_shutdown(struct nsd *nsd)
 	/* SERVER: close command channels to children */
 	if(!nsd->this_child)
 	{
-		for(i=0; i<nsd->child_count; ++i)
+		for(i=0; i < nsd->child_count; ++i)
 			if(nsd->children[i].child_fd > 0)
 			{
 				close(nsd->children[i].child_fd);
@@ -602,7 +608,8 @@ server_start_xfrd(struct nsd *nsd, netio_handler_type* handler)
 		zone->updated = 0;
 	}
 
-	if(handler->fd != -1) close(handler->fd);
+	if(handler->fd != -1)
+		close(handler->fd);
 	if (socketpair(AF_UNIX, SOCK_STREAM, 0, sockets) == -1) {
 		log_msg(LOG_ERR, "startxfrd failed on socketpair: %s", strerror(errno));
 		return -1;
@@ -613,17 +620,18 @@ server_start_xfrd(struct nsd *nsd, netio_handler_type* handler)
 		log_msg(LOG_ERR, "fork xfrd failed: %s", strerror(errno));
 		break;
 	case 0:
-		/* CHILD */
+		/* CHILD: close first socket, use second one */
 		close(sockets[0]);
 		xfrd_init(sockets[1], nsd);
 		/* ENOTREACH */
 		break;
 	default:
-		/* PARENT */
+		/* PARENT: close second socket, use first one */
 		close(sockets[1]);
 		handler->fd = sockets[0];
 		break;
 	}
+	/* PARENT only */
 	handler->timeout = NULL;
 	handler->event_types = NETIO_EVENT_READ;
 	handler->event_handler = parent_handle_xfrd_command;
@@ -634,7 +642,7 @@ server_start_xfrd(struct nsd *nsd, netio_handler_type* handler)
 }
 
 /* pass timeout=-1 for blocking. Returns size, 0, -1(err), or -2(timeout) */
-static ssize_t 
+static ssize_t
 block_read(struct nsd* nsd, int s, void* p, ssize_t sz, int timeout)
 {
 	uint8_t* buf = (uint8_t*) p;
@@ -717,21 +725,22 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 		}
 	}
 	if(!diff_read_file(nsd->db, nsd->options, NULL, nsd->child_count)) {
-			log_msg(LOG_ERR, "unable to load the diff file: %s", nsd->options->difffile);
-			exit(1);
+		log_msg(LOG_ERR, "unable to load the diff file: %s", nsd->options->difffile);
+		exit(1);
 	}
 	log_msg(LOG_INFO, "memory recyclebin holds %lu bytes", (unsigned long)
 		region_get_recycle_size(nsd->db->region));
 #ifndef NDEBUG
 	if(nsd_debug_level >= 1)
 		region_log_stats(nsd->db->region);
-#endif
+#endif /* NDEBUG */
 #ifdef NSEC3
 	prehash(nsd->db, 1);
-#endif
+#endif /* NSEC3 */
 
 	initialize_dname_compression_tables(nsd);
 
+	/* <matthijs> get our new process id */
 	old_pid = nsd->pid;
 	nsd->pid = getpid();
 
@@ -741,12 +750,17 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 	set_bind8_alarm(nsd);
 #endif
 
+	/* <matthijs> Start new child processes */
 	if (server_start_children(nsd, server_region, netio, xfrd_sock_p) != 0) {
 		send_children_quit(nsd);
 		exit(1);
 	}
 
-	/* Overwrite pid before closing old parent. */
+	/* Overwrite pid before closing old parent, to avoid race condition:
+	 * - parent process already closed
+	 * - pidfile still contains old_pid
+	 * - control script contacts parent process, using contents of pidfile
+	 */
 	if (writepid(nsd) == -1) {
 		log_msg(LOG_ERR, "cannot overwrite the pidfile %s: %s", nsd->pidfile, strerror(errno));
 	}
@@ -762,7 +776,7 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 		}
 		/* blocking: wait for parent to really quit. (it sends RELOAD as ack) */
 		DEBUG(DEBUG_IPC,1, (LOG_INFO, "reload: ipc wait for ack main"));
-		ret = block_read(nsd, cmdsocket, &cmd, sizeof(cmd), 
+		ret = block_read(nsd, cmdsocket, &cmd, sizeof(cmd),
 			RELOAD_SYNC_TIMEOUT);
 		if(ret == -2) {
 			DEBUG(DEBUG_IPC, 1, (LOG_ERR, "reload timeout QUITSYNC. retry"));
@@ -802,7 +816,7 @@ server_reload(struct nsd *nsd, region_type* server_region, netio_type* netio,
 		/* use blocking writes */
 		if(!write_socket(xfrd_sock, &cmd,  sizeof(cmd)) ||
 			!write_socket(xfrd_sock, &sz, sizeof(sz)) ||
-			!write_socket(xfrd_sock, domain_dname(zone->apex), 
+			!write_socket(xfrd_sock, domain_dname(zone->apex),
 				dname_total_size(domain_dname(zone->apex))))
 		{
 			log_msg(LOG_ERR, "problems sending soa info from reload %d to xfrd: %s",
@@ -901,26 +915,34 @@ server_main(struct nsd *nsd)
 	pid_t reload_pid = -1;
 	pid_t xfrd_pid = -1;
 	sig_atomic_t mode;
-	
+
+	/* <matthijs> assure we are the main process */
 	assert(nsd->server_kind == NSD_SERVER_MAIN);
 
 	xfrd_listener.user_data = (struct ipc_handler_conn_data*)region_alloc(
 		server_region, sizeof(struct ipc_handler_conn_data));
 	xfrd_listener.fd = -1;
 	((struct ipc_handler_conn_data*)xfrd_listener.user_data)->nsd = nsd;
-	((struct ipc_handler_conn_data*)xfrd_listener.user_data)->conn = 
+	((struct ipc_handler_conn_data*)xfrd_listener.user_data)->conn =
 		xfrd_tcp_create(server_region);
+
+	/* <matthijs> start the XFRD process */
 	xfrd_pid = server_start_xfrd(nsd, &xfrd_listener);
 	netio_add_handler(netio, &xfrd_listener);
+
+	/* <matthijs> start the child processes that handle incoming queries */
 	if (server_start_children(nsd, server_region, netio, &xfrd_listener.fd) != 0) {
 		send_children_quit(nsd);
 		exit(1);
 	}
 	reload_listener.fd = -1;
+
+	/* <matthijs> this_child MUST be 0, because this is the parent process */
 	assert(nsd->this_child == 0);
 
+	/* <matthijs> run the server until we get a shutdown signal */
 	while ((mode = nsd->mode) != NSD_SHUTDOWN) {
-
+		/* <matthijs> did we receive a signal that changes our mode? */
 		if(mode == NSD_RUN) {
 			nsd->mode = mode = server_signal_mode(nsd);
 		}
@@ -938,7 +960,7 @@ server_main(struct nsd *nsd)
 					log_msg(LOG_WARNING,
 					       "server %d died unexpectedly with status %d, restarting",
 					       (int) child_pid, status);
-					restart_child_servers(nsd, server_region, netio, 
+					restart_child_servers(nsd, server_region, netio,
 						&xfrd_listener.fd);
 				} else if (child_pid == reload_pid) {
 					sig_atomic_t cmd = NSD_SOA_END;
@@ -950,11 +972,11 @@ server_main(struct nsd *nsd)
 					reload_listener.fd = -1;
 					reload_listener.event_types = NETIO_EVENT_NONE;
 					/* inform xfrd reload attempt ended */
-					if(!write_socket(xfrd_listener.fd, 
-						&cmd, sizeof(cmd)) == -1) { 
+					if(!write_socket(xfrd_listener.fd,
+						&cmd, sizeof(cmd)) == -1) {
 						log_msg(LOG_ERR, "problems "
-						  "sending SOAEND to xfrd: %s", 
-						  strerror(errno)); 
+						  "sending SOAEND to xfrd: %s",
+						  strerror(errno));
 					}
 				} else if (child_pid == xfrd_pid) {
 					log_msg(LOG_WARNING,
@@ -989,6 +1011,7 @@ server_main(struct nsd *nsd)
 
 			break;
 		case NSD_RELOAD:
+			/* <matthijs> continue to run nsd after reload */
 			nsd->mode = NSD_RUN;
 
 			if (reload_pid != -1) {
@@ -1004,6 +1027,8 @@ server_main(struct nsd *nsd)
 				reload_pid = -1;
 				break;
 			}
+
+			/* <matthijs> do actual reload */
 			reload_pid = fork();
 			switch (reload_pid) {
 			case -1:
@@ -1026,7 +1051,9 @@ server_main(struct nsd *nsd)
 				DEBUG(DEBUG_IPC,2, (LOG_INFO, "Reload resetup; run"));
 				break;
 			default:
-				/* PARENT */
+				/* PARENT, keep running until NSD_QUIT_SYNC
+				 * received from CHILD.
+				 */
 				close(reload_sockets[1]);
 				reload_listener.fd = reload_sockets[0];
 				reload_listener.timeout = NULL;
@@ -1037,12 +1064,12 @@ server_main(struct nsd *nsd)
 				break;
 			}
 			break;
-		case NSD_QUIT_SYNC: 
+		case NSD_QUIT_SYNC:
 			/* synchronisation of xfrd, parent and reload */
 			if(!nsd->quit_sync_done && reload_listener.fd > 0) {
 				sig_atomic_t cmd = NSD_RELOAD;
 				/* stop xfrd ipc writes in progress */
-				DEBUG(DEBUG_IPC,1, (LOG_INFO, 
+				DEBUG(DEBUG_IPC,1, (LOG_INFO,
 					"main: ipc send indication reload"));
 				if(!write_socket(xfrd_listener.fd, &cmd, sizeof(cmd))) {
 					log_msg(LOG_ERR, "server_main: could not send reload "
@@ -1054,7 +1081,7 @@ server_main(struct nsd *nsd)
 			}
 			nsd->mode = NSD_RUN;
 			break;
-		case NSD_QUIT: 
+		case NSD_QUIT:
 			/* silent shutdown during reload */
 			if(reload_listener.fd > 0) {
 				/* acknowledge the quit, to sync reload that we will really quit now */
@@ -1102,14 +1129,15 @@ server_main(struct nsd *nsd)
 	/* Unlink it if possible... */
 	unlink(nsd->pidfile);
 
-	if(reload_listener.fd > 0) close(reload_listener.fd);
+	if(reload_listener.fd > 0)
+		close(reload_listener.fd);
 	if(xfrd_listener.fd > 0) {
 		/* complete quit, stop xfrd */
 		sig_atomic_t cmd = NSD_QUIT;
-		DEBUG(DEBUG_IPC,1, (LOG_INFO, 
+		DEBUG(DEBUG_IPC,1, (LOG_INFO,
 			"main: ipc send quit to xfrd"));
 		if(!write_socket(xfrd_listener.fd, &cmd, sizeof(cmd))) {
-			log_msg(LOG_ERR, "server_main: could not send quit to xfrd: %s", 
+			log_msg(LOG_ERR, "server_main: could not send quit to xfrd: %s",
 				strerror(errno));
 		}
 		fsync(xfrd_listener.fd);
@@ -1137,16 +1165,17 @@ server_child(struct nsd *nsd)
 	netio_type *netio = netio_create(server_region);
 	netio_handler_type *tcp_accept_handlers;
 	sig_atomic_t mode;
-	
+
 	assert(nsd->server_kind != NSD_SERVER_MAIN);
-	
+	DEBUG(DEBUG_IPC, 2, (LOG_INFO, "child process started"));
+
 	if (!(nsd->server_kind & NSD_SERVER_TCP)) {
 		close_all_sockets(nsd->tcp, nsd->ifs);
 	}
 	if (!(nsd->server_kind & NSD_SERVER_UDP)) {
 		close_all_sockets(nsd->udp, nsd->ifs);
 	}
-	
+
 	if (nsd->this_child && nsd->this_child->parent_fd != -1) {
 		netio_handler_type *handler;
 
@@ -1173,7 +1202,7 @@ server_child(struct nsd *nsd)
 				server_region,
 				sizeof(struct udp_handler_data));
 			data->query = query_create(
-				server_region, compressed_dname_offsets, 
+				server_region, compressed_dname_offsets,
 				compression_table_size);
 			data->nsd = nsd;
 			data->socket = &nsd->udp[i];
@@ -1200,7 +1229,7 @@ server_child(struct nsd *nsd)
 		for (i = 0; i < nsd->ifs; ++i) {
 			struct tcp_accept_handler_data *data;
 			netio_handler_type *handler;
-			
+
 			data = (struct tcp_accept_handler_data *) region_alloc(
 				server_region,
 				sizeof(struct tcp_accept_handler_data));
@@ -1208,7 +1237,7 @@ server_child(struct nsd *nsd)
 			data->socket = &nsd->tcp[i];
 			data->tcp_accept_handler_count = nsd->ifs;
 			data->tcp_accept_handlers = tcp_accept_handlers;
-			
+
 			handler = &tcp_accept_handlers[i];
 			handler->fd = nsd->tcp[i].s;
 			handler->timeout = NULL;
@@ -1218,8 +1247,8 @@ server_child(struct nsd *nsd)
 			netio_add_handler(netio, handler);
 		}
 	}
-	
-	/* The main loop... */	
+
+	/* The main loop... */
 	while ((mode = nsd->mode) != NSD_QUIT) {
 		if(mode == NSD_RUN) nsd->mode = mode = server_signal_mode(nsd);
 
@@ -1228,7 +1257,7 @@ server_child(struct nsd *nsd)
 #ifdef BIND8_STATS
 			/* Dump the statistics */
 			bind8_stats(nsd);
-#else /* BIND8_STATS */
+#else /* !BIND8_STATS */
 			log_msg(LOG_NOTICE, "Statistics support not enabled at compile time.");
 #endif /* BIND8_STATS */
 
@@ -1239,13 +1268,14 @@ server_child(struct nsd *nsd)
 			if (nsd->this_child->parent_fd > 0) {
 				sig_atomic_t parent_notify = NSD_REAP_CHILDREN;
 				if (write(nsd->this_child->parent_fd,
-				    &parent_notify, 
+				    &parent_notify,
 				    sizeof(parent_notify)) == -1)
 				{
 					log_msg(LOG_ERR, "problems sending command from %d to parent: %s",
 						(int) nsd->this_child->pid, strerror(errno));
 				}
-			} else while (waitpid(0, NULL, WNOHANG) > 0) /* no parent, so reap 'em */;
+			} else /* no parent, so reap 'em */
+				while (waitpid(0, NULL, WNOHANG) > 0) ;
 			nsd->mode = NSD_RUN;
 		}
 		else if(mode == NSD_RUN) {
@@ -1259,7 +1289,7 @@ server_child(struct nsd *nsd)
 		} else if(mode == NSD_QUIT) {
 			/* ignore here, quit */
 		} else {
-			log_msg(LOG_ERR, "mode bad value %d, back to service.", 
+			log_msg(LOG_ERR, "mode bad value %d, back to service.",
 				mode);
 			nsd->mode = NSD_RUN;
 		}
@@ -1270,7 +1300,7 @@ server_child(struct nsd *nsd)
 #endif /* BIND8_STATS */
 
 	region_destroy(server_region);
-	
+
 	server_shutdown(nsd);
 }
 
@@ -1284,11 +1314,11 @@ handle_udp(netio_type *ATTR_UNUSED(netio),
 		= (struct udp_handler_data *) handler->user_data;
 	int received, sent;
 	struct query *q = data->query;
-	
+
 	if (!(event_types & NETIO_EVENT_READ)) {
 		return;
 	}
-	
+
 	/* Account... */
 	if (data->socket->addr->ai_family == AF_INET) {
 		STATUP(data->nsd, qudp);
@@ -1412,14 +1442,9 @@ handle_tcp_reading(netio_type *netio,
 				return;
 			} else {
 #ifdef ECONNRESET
-				if (verbosity >= 2 || errno != ECONNRESET) {
-					log_msg(LOG_ERR, "failed reading from tcp: %s", strerror(errno));
-				}
-#else /* ECONNRESET */
-				if (verbosity >= 2) {
-					log_msg(LOG_ERR, "failed reading from tcp: %s", strerror(errno));
-				}
-#endif
+				if (verbosity >= 2 || errno != ECONNRESET)
+#endif /* ECONNRESET */
+				log_msg(LOG_ERR, "failed reading from tcp: %s", strerror(errno));
 				cleanup_tcp_handler(netio, handler);
 				return;
 			}
@@ -1480,14 +1505,9 @@ handle_tcp_reading(netio_type *netio,
 			return;
 		} else {
 #ifdef ECONNRESET
-			if (verbosity >= 2 || errno != ECONNRESET) {
-				log_msg(LOG_ERR, "failed reading from tcp: %s", strerror(errno));
-			}
-#else /* ECONNRESET */
-			if (verbosity >= 2) {
-				log_msg(LOG_ERR, "failed reading from tcp: %s", strerror(errno));
-			}
-#endif
+			if (verbosity >= 2 || errno != ECONNRESET)
+#endif /* ECONNRESET */
+			log_msg(LOG_ERR, "failed reading from tcp: %s", strerror(errno));
 			cleanup_tcp_handler(netio, handler);
 			return;
 		}
@@ -1536,18 +1556,18 @@ handle_tcp_reading(netio_type *netio,
 	{
 		STATUP(data->nsd, nona);
 	}
-		
+
 	query_add_optional(data->query, data->nsd);
 
 	/* Switch to the tcp write handler.  */
 	buffer_flip(data->query->packet);
 	data->query->tcplen = buffer_remaining(data->query->packet);
 	data->bytes_transmitted = 0;
-	
+
 	handler->timeout->tv_sec = TCP_TIMEOUT;
 	handler->timeout->tv_nsec = 0L;
 	timespec_add(handler->timeout, netio_current_time(netio));
-	
+
 	handler->event_types = NETIO_EVENT_WRITE | NETIO_EVENT_TIMEOUT;
 	handler->event_handler = handle_tcp_writing;
 }
@@ -1585,8 +1605,8 @@ handle_tcp_writing(netio_type *netio,
 				return;
 			} else {
 #ifdef ECONNRESET
-				if(verbosity>=2 || errno != ECONNRESET)
-#endif
+				if(verbosity >= 2 || errno != ECONNRESET)
+#endif /* ECONNRESET */
 				log_msg(LOG_ERR, "failed writing to tcp: %s", strerror(errno));
 				cleanup_tcp_handler(netio, handler);
 				return;
@@ -1619,8 +1639,8 @@ handle_tcp_writing(netio_type *netio,
 			return;
 		} else {
 #ifdef ECONNRESET
-			if(verbosity>=2 || errno != ECONNRESET)
-#endif
+			if(verbosity >= 2 || errno != ECONNRESET)
+#endif /* ECONNRESET */
 			log_msg(LOG_ERR, "failed writing to tcp: %s", strerror(errno));
 			cleanup_tcp_handler(netio, handler);
 			return;
@@ -1645,7 +1665,7 @@ handle_tcp_writing(netio_type *netio,
 		data->query_state = query_axfr(data->nsd, q);
 		if (data->query_state != QUERY_PROCESSED) {
 			query_add_optional(data->query, data->nsd);
-			
+
 			/* Reset data. */
 			buffer_flip(q->packet);
 			q->tcplen = buffer_remaining(q->packet);
@@ -1668,7 +1688,7 @@ handle_tcp_writing(netio_type *netio,
 	 * TCP socket by installing the TCP read handler.
 	 */
 	data->bytes_transmitted = 0;
-	
+
 	handler->timeout->tv_sec = TCP_TIMEOUT;
 	handler->timeout->tv_nsec = 0;
 	timespec_add(handler->timeout, netio_current_time(netio));
@@ -1700,7 +1720,7 @@ handle_tcp_accept(netio_type *netio,
 	struct sockaddr_in addr;
 #endif
 	socklen_t addrlen;
-	
+
 	if (!(event_types & NETIO_EVENT_READ)) {
 		return;
 	}
@@ -1708,17 +1728,17 @@ handle_tcp_accept(netio_type *netio,
 	if (data->nsd->current_tcp_count >= data->nsd->maximum_tcp_count) {
 		return;
 	}
-	
+
 	/* Accept it... */
 	addrlen = sizeof(addr);
 	s = accept(handler->fd, (struct sockaddr *) &addr, &addrlen);
 	if (s == -1) {
 		/* EINTR is a signal interrupt. The others are various OS ways
 		   of saying that the client has closed the connection. */
-		if (	errno != EINTR 
-			&& errno != EWOULDBLOCK 
+		if (	errno != EINTR
+			&& errno != EWOULDBLOCK
 #ifdef ECONNABORTED
-			&& errno != ECONNABORTED 
+			&& errno != ECONNABORTED
 #endif /* ECONNABORTED */
 #ifdef EPROTO
 			&& errno != EPROTO
@@ -1734,7 +1754,7 @@ handle_tcp_accept(netio_type *netio,
 		close(s);
 		return;
 	}
-	
+
 	/*
 	 * This region is deallocated when the TCP connection is
 	 * closed by the TCP handler.
@@ -1746,7 +1766,7 @@ handle_tcp_accept(netio_type *netio,
 	tcp_data->query = query_create(tcp_region, compressed_dname_offsets,
 		compression_table_size);
 	tcp_data->nsd = data->nsd;
-	
+
 	tcp_data->tcp_accept_handler_count = data->tcp_accept_handler_count;
 	tcp_data->tcp_accept_handlers = data->tcp_accept_handlers;
 
@@ -1754,7 +1774,7 @@ handle_tcp_accept(netio_type *netio,
 	tcp_data->bytes_transmitted = 0;
 	memcpy(&tcp_data->query->addr, &addr, addrlen);
 	tcp_data->query->addrlen = addrlen;
-	
+
 	tcp_handler = (netio_handler_type *) region_alloc(
 		tcp_region, sizeof(netio_handler_type));
 	tcp_handler->fd = s;
@@ -1783,7 +1803,7 @@ handle_tcp_accept(netio_type *netio,
 	}
 }
 
-static void 
+static void
 send_children_quit(struct nsd* nsd)
 {
 	sig_atomic_t command = NSD_QUIT;
@@ -1808,7 +1828,7 @@ send_children_quit(struct nsd* nsd)
 	}
 }
 
-static void 
+static void
 set_children_stats(struct nsd* nsd)
 {
 	size_t i;
@@ -1828,7 +1848,7 @@ configure_handler_event_types(size_t count,
 	size_t i;
 
 	assert(handlers);
-	
+
 	for (i = 0; i < count; ++i) {
 		handlers[i].event_types = event_types;
 	}
