@@ -12,9 +12,11 @@
 #if defined(TSIG) && defined(HAVE_SSL)
 
 #include <openssl/hmac.h>
+#include <openssl/sha.h>
 
 #include "tsig-openssl.h"
 #include "tsig.h"
+#include "util.h"
 
 static void *create_context(region_type *region);
 static void init_context(void *context,
@@ -23,36 +25,53 @@ static void init_context(void *context,
 static void update(void *context, const void *data, size_t size);
 static void final(void *context, uint8_t *digest, size_t *size);
 
+static int
+tsig_openssl_init_algorithm(region_type* region,
+	const char* digest, const char* name, const char* wireformat)
+{
+	tsig_algorithm_type* algorithm;
+	const EVP_MD *hmac_algorithm;
+
+	hmac_algorithm = EVP_get_digestbyname(digest);
+	if (!hmac_algorithm) {
+		log_msg(LOG_ERR, "%s digest not available", digest);
+		return 0;
+	}
+
+	algorithm = (tsig_algorithm_type *) region_alloc(
+		region, sizeof(tsig_algorithm_type));
+	algorithm->short_name = name;
+	algorithm->wireformat_name
+		= dname_parse(region, wireformat);
+	if (!algorithm->wireformat_name) {
+		log_msg(LOG_ERR, "cannot parse %s algorithm", wireformat);
+		return 0;
+	}
+	algorithm->maximum_digest_size = EVP_MAX_MD_SIZE;
+	algorithm->data = hmac_algorithm;
+	algorithm->hmac_create_context = create_context;
+	algorithm->hmac_init_context = init_context;
+	algorithm->hmac_update = update;
+	algorithm->hmac_final = final;
+	tsig_add_algorithm(algorithm);
+
+	return 1;
+}
+
 int
 tsig_openssl_init(region_type *region)
 {
-	tsig_algorithm_type *md5_algorithm;
-	const EVP_MD *hmac_md5_algorithm;
-
 	OpenSSL_add_all_digests();
-	hmac_md5_algorithm = EVP_get_digestbyname("md5");
-	if (!hmac_md5_algorithm) {
-		log_msg(LOG_ERR, "hmac-md5 algorithm not available");
-		return 0;
-	}
 
-	md5_algorithm = (tsig_algorithm_type *) region_alloc(
-		region, sizeof(tsig_algorithm_type));
-	md5_algorithm->short_name = "hmac-md5";
-	md5_algorithm->wireformat_name
-		= dname_parse(region, "hmac-md5.sig-alg.reg.int.");
-	if (!md5_algorithm->wireformat_name) {
- 		log_msg(LOG_ERR, "cannot parse MD5 algorithm name");
+	/* TODO: walk lookup supported algorithms table */
+	if (!tsig_openssl_init_algorithm(region, "md5", "hmac-md5","hmac-md5.sig-alg.reg.int."))
 		return 0;
-	}
-	md5_algorithm->maximum_digest_size = EVP_MAX_MD_SIZE;
-	md5_algorithm->data = hmac_md5_algorithm;
-	md5_algorithm->hmac_create_context = create_context;
-	md5_algorithm->hmac_init_context = init_context;
-	md5_algorithm->hmac_update = update;
-	md5_algorithm->hmac_final = final;
-	tsig_add_algorithm(md5_algorithm);
-	
+	if (!tsig_openssl_init_algorithm(region, "sha1", "hmac-sha1", "hmac-sha1."))
+		return 0;
+#ifdef SHA256_DIGEST_LENGTH
+	if (!tsig_openssl_init_algorithm(region, "sha256", "hmac-sha256", "hmac-sha256."))
+		return 0;
+#endif
 	return 1;
 }
 
